import { Switch, Route, useLocation } from "wouter";
import { queryClient } from "./lib/queryClient";
import { QueryClientProvider } from "@tanstack/react-query";
import { Toaster } from "@/components/ui/toaster";
import { lazy, Suspense, useState, useEffect, useCallback, memo } from "react";
import { initAnalytics, trackPageView } from "@/lib/analytics";
import { CookieConsent } from "@/components/ui/cookie-consent";
import { Header } from "@/components/layout/header";
import { Footer } from "@/components/layout/footer";
import { MobileNavigation } from "@/components/layout/mobile-navigation";
import { BasketProvider } from "@/contexts/BasketContext";
import HashHandler from "@/components/helpers/hash-handler";

// Eagerly loaded components for critical paths only
import NotFound from "@/pages/not-found";

// Implement dynamic imports with priority loading hints
// Higher priority routes (preload: true) will be loaded earlier
type LazyImport = () => Promise<any>;

interface LazyComponentWithPreload extends React.LazyExoticComponent<React.ComponentType<any>> {
  preload?: () => void;
}

const lazyWithPreload = (factory: LazyImport, preload = false): LazyComponentWithPreload => {
  const Component = lazy(factory) as LazyComponentWithPreload;
  
  // Attach preload function to component
  Component.preload = () => {
    // Start loading the component in the background
    factory();
  };
  
  // Preload immediately if this is a critical component
  if (preload && Component.preload) {
    Component.preload();
  }
  
  return Component;
};

// Optimize main entry point with higher priority
const HomePage = lazyWithPreload(() => import("@/pages/home"), true);
const SitemapPage = lazyWithPreload(() => import("@/pages/sitemap"));

// Group lazy-loaded components by usage patterns
// Main site components (higher priority)
const LandingPage = lazyWithPreload(() => import("@/pages/landing"));
const MiqeyIntegrationPage = lazyWithPreload(() => import("@/pages/miqey-integration"));
const BuildingsPage = lazyWithPreload(() => import("@/pages/buildings"));
const BookingsPage = lazyWithPreload(() => import("@/pages/bookings"));

// E-commerce components (medium priority)
const BasketPage = lazyWithPreload(() => import("@/pages/basket"));
const WebshopPage = lazyWithPreload(() => import("@/pages/webshop"));
const ProductDetailPage = lazyWithPreload(() => import("@/pages/product-detail"));
const CheckoutPage = lazyWithPreload(() => import("@/pages/checkout"));

// Order management (lower priority)
const OrderConfirmationPage = lazyWithPreload(() => import("@/pages/order-confirmation"));
const OrderSuccessPage = lazyWithPreload(() => import("@/pages/order-success"));
const OrderCancelledPage = lazyWithPreload(() => import("@/pages/order-cancelled"));
const OrderHistoryPage = lazyWithPreload(() => import("@/pages/order-history"));
const OrderDetailPage = lazyWithPreload(() => import("@/pages/order-detail"));
const AccountPage = lazyWithPreload(() => import("@/pages/account"));

// Admin and dashboard components (low priority - load last)
const AdminPage = lazyWithPreload(() => import("@/pages/admin"));
const DashboardPage = lazyWithPreload(() => import("@/pages/dashboard"));
const DisplaysPage = lazyWithPreload(() => import("@/pages/displays"));
const AnalyticsPage = lazyWithPreload(() => import("@/pages/analytics"));
const SettingsPage = lazyWithPreload(() => import("@/pages/settings"));
const SubscriptionManagement = lazyWithPreload(() => import("@/pages/subscription-management"));
const PartnerPortalPage = lazyWithPreload(() => import("@/pages/partner-portal"));
const WorkspaceAnalyticsPage = lazyWithPreload(() => import("@/pages/workspace-analytics"));
const ClientWorkspaceAnalyticsPage = lazyWithPreload(() => import("@/pages/client-workspace-analytics"));
const ClientInsightsPage = lazyWithPreload(() => import("@/pages/client-insights"));

// Content pages (lower priority)
const MarketingPage = lazyWithPreload(() => import("@/pages/marketing"));
const BlogPage = lazyWithPreload(() => import("@/pages/blog"));
const BlogListingPage = lazyWithPreload(() => import("@/pages/blog-listing"));
const BlogDetailPage = lazyWithPreload(() => import("@/pages/blog-detail"));
const BlogCorporatePage = lazyWithPreload(() => import("@/pages/blog-corporate"));
const BlogRevenuePage = lazyWithPreload(() => import("@/pages/blog-revenue"));
const PlanGeneratorPage = lazyWithPreload(() => import("@/pages/plan-generator"));
const OnboardingPage = lazyWithPreload(() => import("@/pages/onboarding-page"));
const FloorPage = lazyWithPreload(() => import("@/pages/floor-page"));
const SignagePage = lazyWithPreload(() => import("@/pages/signage"));
const ApiReferencePage = lazy(() => import("@/pages/api-reference"));
const WebhooksPage = lazy(() => import("@/pages/webhooks"));

// Empty loader for mobile-first loading without visible spinners
const MobileOptimizedLoader = () => null;

// Hidden loader that only shows on desktop after a delay
const PageLoader = () => (
  <div className="hidden md:flex justify-center items-center min-h-screen opacity-0 transition-opacity duration-300 delay-800 animate-delayed-fade-in">
    <div className="animate-spin w-8 h-8 border-4 border-primary border-t-transparent rounded-full" />
  </div>
);

// Page title mapping
const PAGE_TITLES: Record<string, string> = {
  "/": "Ellie - Smart Meeting Room Solutions for Microsoft Teams",
  "/blog": "Ellie - Simplify Workplace Collaboration with Zero Technical Frustration",
  "/blog-corporate": "Ellie - The Hidden Truth About Office Space Utilization",
  "/blog-revenue": "Ellie - Unlock More Revenue from Your Meeting Rooms with Ellie",
  "/blogs": "Ellie - Blog Articles and Resources",
  "/workspace-analytics": "Ellie - Workspace Utilization Analytics Dashboard",
  "/plan-generator": "Ellie - Interactive Floor Plan Generator",
  "/miqey": "Ellie - Miqey API Integration",
  "/signage": "Ellie - Digital Signage Management",
  "/dashboard": "Ellie - Dashboard",
  "/displays": "Ellie - Display Management",
  "/analytics": "Ellie - Analytics",
  "/settings": "Ellie - Settings",
  "/subscriptions": "Ellie - Subscription Management",
  "/subscription-management": "Ellie - Subscription Management",
  "/partner": "Ellie - Partner Portal",
  "/admin": "Ellie - Admin Panel",
  "/onboarding": "Ellie - Onboarding",
  "/client-insights": "Ellie - Client Portal Insights",
  "/webshop": "Ellie - Shop Devices and Licenses",
  "/product/:id": "Ellie - Product Details",
  "/checkout": "Ellie - Secure Checkout",
  "/basket": "Ellie - Your Shopping Basket",
  "/account": "Ellie - Your Account",
  "/order-confirmation": "Ellie - Order Confirmation",
  "/order-success": "Ellie - Order Successful",
  "/order-cancelled": "Ellie - Order Cancelled",
  "/order-history": "Ellie - Your Order History",
  "/order/:id": "Ellie - Order Details",
  "/sitemap": "Ellie - Site Map",
  "/api-reference": "Ellie - API Reference Documentation",
  "/webhooks": "Ellie - Webhooks Documentation",
  "/bookings": "Ellie - Meeting Room Booking"
};

// Default fallback title
const DEFAULT_TITLE = "Ellie - Page Not Found";

// Initialize GA4 tracking
// Replace with your actual GA4 Measurement ID when deploying
const GA_MEASUREMENT_ID = "G-XXXXXXXXXX";

// Memoized Router component to prevent unnecessary re-renders
const Router = memo(function Router() {
  const [location] = useLocation();
  const [analyticsEnabled, setAnalyticsEnabled] = useState<boolean>(false);
  
  // Initialize analytics with consent handling
  useEffect(() => {
    const hasConsent = localStorage.getItem('cookie_consent') === 'accepted';
    if (hasConsent) {
      initAnalytics(GA_MEASUREMENT_ID);
      setAnalyticsEnabled(true);
    }
  }, []);
  
  // Track page views when location changes (if consent given)
  useEffect(() => {
    if (analyticsEnabled) {
      const title = getPageTitle(location);
      trackPageView(location, title);
      
      // Update document title
      document.title = title;
    }
    
    // Scroll to top when location changes
    window.scrollTo(0, 0);
  }, [location, analyticsEnabled]);
  
  // Memoized helper function to get page title
  const getPageTitle = useCallback((path: string): string => {
    // Handle dynamic paths like /floor/:id or /landing/:audienceId
    if (path.startsWith('/floor/')) return "Ellie - Floor Management";
    if (path.startsWith('/landing/')) return "Ellie - Welcome";
    if (path.startsWith('/order/') && path !== '/order-history' && 
        path !== '/order-success' && path !== '/order-cancelled' && 
        path !== '/order-confirmation') 
      return "Ellie - Order Details";
    
    // Return from mapping or default
    return PAGE_TITLES[path] || DEFAULT_TITLE;
  }, []);
  
  // Memoized cookie handlers
  const handleAcceptCookies = useCallback(() => {
    initAnalytics(GA_MEASUREMENT_ID);
    setAnalyticsEnabled(true);
    // Track initial page view when consent is given
    const title = getPageTitle(location);
    trackPageView(location, title);
  }, [location, getPageTitle]);
  
  const handleRejectCookies = useCallback(() => {
    setAnalyticsEnabled(false);
    localStorage.setItem('cookie_consent', 'rejected');
  }, []);
  
  // Determine if we're on a landing page or onboarding page (where header should be hidden)
  const hideHeader = location.startsWith('/landing/') || location === '/onboarding';
  
  // Pages that use their own layout component (with header and footer)
  const hasCustomLayout = location === '/api-reference' || location === '/webhooks';
  
  return (
    <div className="flex flex-col min-h-screen overflow-x-hidden">
      {/* Add HashHandler to handle URL hash navigation */}
      <HashHandler />
      
      {!hideHeader && !hasCustomLayout && (
        <div className="header-container" data-prioritize="true">
          <Header />
        </div>
      )}
      <main className="flex-grow w-full relative">
        <Suspense fallback={<div className="hidden" />}>
          <Switch>
            <Route path="/" component={HomePage} />
            <Route path="/admin" component={AdminPage} />
            <Route path="/marketing" component={MarketingPage} />
            <Route path="/dashboard" component={DashboardPage} />
            <Route path="/displays" component={DisplaysPage} />
            <Route path="/analytics" component={AnalyticsPage} />
            <Route path="/settings" component={SettingsPage} />
            <Route path="/subscriptions" component={SubscriptionManagement} />
            <Route path="/subscription-management" component={SubscriptionManagement} />
            <Route path="/partner" component={PartnerPortalPage} />
            <Route path="/blogs" component={BlogListingPage} />
            <Route path="/blog" component={BlogPage} />
            <Route path="/blog-corporate" component={BlogDetailPage} />
            <Route path="/blog-revenue" component={BlogDetailPage} />
            <Route path="/blog-collaboration" component={BlogDetailPage} />
            <Route path="/blog-productivity" component={BlogDetailPage} />
            <Route path="/blog-roi" component={BlogDetailPage} />
            <Route path="/blog-teams" component={BlogDetailPage} />
            <Route path="/blog-it" component={BlogDetailPage} />
            <Route path="/blog-guide" component={BlogDetailPage} />
            <Route path="/workspace-analytics" component={WorkspaceAnalyticsPage} />
            <Route path="/client-workspace-analytics" component={ClientWorkspaceAnalyticsPage} />
            <Route path="/plan-generator" component={PlanGeneratorPage} />
            <Route path="/onboarding" component={OnboardingPage} />
            <Route path="/floor/:id" component={FloorPage} />
            <Route path="/landing/:audienceId" component={LandingPage} />
            <Route path="/miqey" component={MiqeyIntegrationPage} />
            <Route path="/signage" component={SignagePage} />
            <Route path="/buildings" component={BuildingsPage} />
            <Route path="/bookings" component={BookingsPage} />
            <Route path="/client-insights" component={ClientInsightsPage} />
            <Route path="/basket" component={BasketPage} />
            <Route path="/webshop" component={WebshopPage} />
            <Route path="/product/:id" component={ProductDetailPage} />
            <Route path="/checkout" component={CheckoutPage} />
            <Route path="/order-confirmation" component={OrderConfirmationPage} />
            <Route path="/order-success" component={OrderSuccessPage} />
            <Route path="/order-cancelled" component={OrderCancelledPage} />
            <Route path="/order-history" component={OrderHistoryPage} />
            <Route path="/order/:id" component={OrderDetailPage} />
            <Route path="/account" component={AccountPage} />
            <Route path="/sitemap" component={SitemapPage} />
            <Route path="/api-reference" component={ApiReferencePage} />
            <Route path="/webhooks" component={WebhooksPage} />
            <Route component={NotFound} />
          </Switch>
        </Suspense>
      </main>
      {!hideHeader && !hasCustomLayout && (
        <div className="footer-container lazy-section">
          <Footer />
        </div>
      )}
      
      {/* Mobile navigation for client portal */}
      <div className="mobile-nav-container" data-prioritize="true">
        <MobileNavigation />
      </div>
      
      {/* Cookie consent banner */}
      <div className="cookie-banner-container lazy-section">
        <CookieConsent
          onAccept={handleAcceptCookies}
          onReject={handleRejectCookies}
        />
      </div>
    </div>
  );
});

// Optimized App component with performance enhancements
function App() {
  // Use useEffect for metrics - execute after hydration
  useEffect(() => {
    // Report web vitals after initial render
    if ('performance' in window) {
      // Use requestIdleCallback for non-critical operations
      const reportWebVitals = () => {
        try {
          // Get and report FCP (First Contentful Paint)
          const fcpEntry = performance.getEntriesByName('first-contentful-paint')[0];
          if (fcpEntry) {
            console.info('FCP:', Math.round(fcpEntry.startTime));
          }
  
          // Get and report LCP (Largest Contentful Paint) if available
          const lcpEntries = performance.getEntriesByType('largest-contentful-paint');
          if (lcpEntries && lcpEntries.length > 0) {
            console.info('LCP:', Math.round(lcpEntries[0].startTime));
          }
        } catch (e) {
          // Silent fail for older browsers
        }
      };
      
      // Schedule metrics reporting during idle time
      if ('requestIdleCallback' in window) {
        (window as any).requestIdleCallback(reportWebVitals);
      } else {
        setTimeout(reportWebVitals, 2000);
      }
    }
  }, []);

  return (
    <QueryClientProvider client={queryClient}>
      <BasketProvider>
        <Router />
        <Toaster />
      </BasketProvider>
    </QueryClientProvider>
  );
}

export default App;
